#!/bin/bash
############################################################################
#                                                                          #
# This file is part of the IPFire Firewall.                                #
#                                                                          #
# IPFire is free software; you can redistribute it and/or modify           #
# it under the terms of the GNU General Public License as published by     #
# the Free Software Foundation; either version 3 of the License, or        #
# (at your option) any later version.                                      #
#                                                                          #
# IPFire is distributed in the hope that it will be useful,                #
# but WITHOUT ANY WARRANTY; without even the implied warranty of           #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
# GNU General Public License for more details.                             #
#                                                                          #
# You should have received a copy of the GNU General Public License        #
# along with IPFire; if not, write to the Free Software                    #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA #
#                                                                          #
# Copyright (C) 2020 IPFire-Team <info@ipfire.org>.                        #
#                                                                          #
############################################################################
#
. /opt/pakfire/lib/functions.sh
/usr/local/bin/backupctrl exclude >/dev/null 2>&1

core=148

# Remove old core updates from pakfire cache to save space...
for (( i=1; i<=$core; i++ )); do
	rm -f /var/cache/pakfire/core-upgrade-*-$i.ipfire
done

# Remove files
rm -vf \
	/etc/rc.d/init.d/networking/red.ip/99-geoip-database \
	/srv/web/ipfire/cgi-bin/geoip-block.cgi \
	/usr/lib/libGeoIP*
	/usr/lib/perl5/site_perl/5.30.0/Geo \
	/usr/share/GeoIP \
	/var/lib/GoIP \
	/usr/local/bin/xt_geoip* \
	/var/ipfire/geoip-functions.pl \

# Stop services

# Extract files
extract_files

# update linker config
ldconfig

# update location database
/usr/local/bin/convert-to-location

# Update Language cache
/usr/local/bin/update-lang-cache

# Filesytem cleanup
/usr/local/bin/filesystem-cleanup

# Start services

# Update crontab
sed -i /var/spool/cron/root.orig \
	-e "s/xt_geoip_update/update-location-database/" \
	-e "/location/s/monthly/hourly/" \
	-e "s/GeoIP/location/"
fcrontab -z

# This update needs a reboot...
#touch /var/run/need_reboot

# Finish
/etc/init.d/fireinfo start
sendprofile

# Update grub config to display new core version
if [ -e /boot/grub/grub.cfg ]; then
	grub-mkconfig -o /boot/grub/grub.cfg
fi

sync

# Don't report the exitcode last command
exit 0
